@@ -2,30 +2,19 @@
 
 // #include <bigstatsr/arma-strict-R-headers.h>
 #include <bigstatsr/BMCodeAcc.h>
+#include "bed-acc.h"
 
 using namespace Rcpp;
 
 /******************************************************************************/
 
-// [[Rcpp::export]]
-List corMat(Environment BM,
-            const IntegerVector& rowInd,
-            const IntegerVector& colInd,
-            double size,
-            const NumericVector& thr,
-            const NumericVector& pos,
-            const NumericVector& info,
-            int ncores) {
-
-  myassert_size(colInd.size(), pos.size());
-  myassert_size(colInd.size(), info.size());
-
-  NumericVector sqrt_info = sqrt(info);
-
-  XPtr<FBM> xpBM = BM["address"];
-  NumericVector code = clone(as<NumericVector>(BM["code256"]));
-  code[is_na(code)] = 3;
-  SubBMCode256Acc macc(xpBM, rowInd, colInd, code, 1);
+template <class C>
+List corMat0(C macc,
+             double size,
+             const NumericVector& thr,
+             const NumericVector& pos,
+             bool fill_diag,
+             int ncores) {
 
   int n = macc.nrow();
   int m = macc.ncol();
@@ -44,6 +33,10 @@ List corMat(Environment BM,
 
       ind.clear();
       val.clear();
+      if (fill_diag) {
+        ind.push_back(j0);
+        val.push_back(1.0);
+      }
 
       // pre-computation
       double xSum0 = 0, xxSum0 = 0;
@@ -85,21 +78,50 @@ List corMat(Environment BM,
         double deno_x = xxSum - xSum * xSum / nona;
         double deno_y = yySum - ySum * ySum / nona;
         double r = num / ::sqrt(deno_x * deno_y);
-        r *= sqrt_info[j] * sqrt_info[j0];
 
-        if (std::abs(r) > thr[nona - 1]) {
-          ind.push_back(j + 1);
+        if (ISNAN(r) || std::abs(r) > thr[nona - 1]) {
+          ind.push_back(j);
           val.push_back(r);
         }
       }
 
       #pragma omp critical
-      res[j0] = List::create(_["i"] = wrap(ind), _["x"] = wrap(val));
+      res[j0] = List::create(
+        _["i"] = rev(as<IntegerVector>(wrap(ind))),
+        _["x"] = rev(as<NumericVector>(wrap(val))));
     }
   }
 
-
   return res;
 }
 
 /******************************************************************************/
+
+// [[Rcpp::export]]
+List corMat(Environment obj,
+            const IntegerVector& rowInd,
+            const IntegerVector& colInd,
+            double size,
+            const NumericVector& thr,
+            const NumericVector& pos,
+            bool fill_diag,
+            int ncores) {
+
+  myassert_size(colInd.size(), pos.size());
+
+  if (obj.exists("code256")) {
+    XPtr<FBM> xpBM = obj["address"];
+    NumericVector code = clone(as<NumericVector>(obj["code256"]));
+    code[is_na(code)] = 3;
+    SubBMCode256Acc macc(xpBM, rowInd, colInd, code, 1);
+    return corMat0(macc, size, thr, pos, fill_diag, ncores);
+  } else if (obj.exists("bedfile")) {
+    XPtr<bed> xp_bed = obj["address"];
+    bedAcc macc(xp_bed, rowInd, colInd);
+    return corMat0(macc, size, thr, pos, fill_diag, ncores);
+  } else {
+    throw Rcpp::exception("Unknown object type.");
+  }
+}
+
+/******************************************************************************/
